clear
octave=0
if (octave==1)
    fig=0;
else
    fig=1;
end
fig=1;

%=============================
% General parameter
%=============================
np=20;  %number of nodes in each dyke
ns=20;  %number of station on each slope


%===============================
% geometry settings
%===============================

%station coordinates
%stations on left slope
a=3/2;
x1=linspace(0,2000,ns);
y1=a*x1;
%stations on right slope
x2=linspace(3000,5000,ns);
y2=3000+a*3000-a*x2;

%plot seismic stations
if (fig==1)
 figure(1);
 plot(x1,y1,'o');
 hold on
 plot(x2,y2,'o');
 axis equal;grid

 %plot slope
 plot([x1(1) x1(end) x2(1) x2(end)],[y1(1) y1(end) y2(1) y2(end)],'k');
end

% left dike node position
xp1=linspace(1000,2500,np+1);
yp1=a*(xp1-1000);
xp1=xp1(1:end-1);
yp1=yp1(1:end-1);
% right dike node position
yp2=yp1;
xp2=4000-yp2/a;

if (fig==1)
 plot(xp1,yp1,'rs');
 plot(xp2,yp2,'gs');

 hold off
end
%
 
%=========================================================
%
% travel time problem
%
%=========================================================

% each dike has a width of DX meter
%
DX=100;
%
% average medium velocity is 2000m/s
Vp=1000;
Sp=1/Vp;
%
% Each dyke maybe cold: Vp=3000m/s or filled with magma: Vp=1000m/s
%
VpLD=zeros(1,np)+3000; %left dyke initialized to 3000
VpRD=zeros(1,np)+3000; %right dyke
SpLD=1./VpLD;
SpRD=1./VpRD;
SpD=[SpLD SpRD]';

%
% The paths available are those running from left to right
% at constant depths, ie ns ttimes
% We build here the matrix giving the Ttimes as function of
% dike velocities.
%
% The ttime is equal to (xp2(i)-xp1(i)-2*DX)*Sp + DX*SpLD + DX*SpRD
% that we write in the vectorial form:
% ttime=ttimeCone + TtimeDyke*SpLD
%
TtimeDyke=zeros(ns,2*np);
ttimeCone=zeros(ns,1);
for i=1:ns            % loop over travel paths
    
    ttimeCone(i)=(xp2(i)-xp1(i)-2*DX)*Sp;
    %
    % current vertical position
    %
    H=y1(i);
    %find dyke parameter at same elevation, or below and above this
    %elevation
    I=find(yp1==H);
    if (length(I)>0)            % dyke node is a the same elevation
        TtimeDyke(i,I(1))=DX;
        TtimeDyke(i,I(1)+np)=DX;
    else
        [DH,I]=sort(abs(H-yp1));
        if (yp1(I(1))>yp1(I(2)))
            i1=I(1);
            i2=I(2);
        else
            i1=I(2);
            i2=I(1);
        end
        a1=(yp1(i1)-H)/(yp1(i1)-yp1(i2));
        a2=(H-yp1(i2))/(yp1(i1)-yp1(i2));
        TtimeDyke(i,i1)=a1*DX;
        TtimeDyke(i,i2)=a2*DX;
        TtimeDyke(i,i1+np)=a1*DX;
        TtimeDyke(i,i2+np)=a2*DX;
    end
end

if (fig==1)
  figure(2);
  plot(y1,TtimeDyke*SpD+ttimeCone,'.');grid
  title('travel time as function of elevation');
  xlabel('meters')
  ylabel('second')
end
    
%=========================================================
%
% gravity problem
%
%=========================================================

%
% gravity stations are located at the same location as seismic station
%
alpha=1018.84/2700/3*100;  % this is a 2D approximation

%
% the gravity in micro gal is given by: g=alpha*Dz/(Dx^2+Dz^2)*(rhoD-2700)/2700
% where Dx and Dy are the difference in horizontal and vertical position
%
% Density in dyke maybe high (2800) when cold, or low (2000) when fluid
% (this is unrealistic)
%
rhoD=zeros(2*np,1)+2800;

%
% recall that a measure is given by: g= alpha*Dz/(Dx^2+Dz^2)/2700*rhoD - alpha*Dz/(Dx^2+Dz^2)
%
% The gravity measurement will be modelled as: Gr*rho + G0
%
% The number of data is now 2*ns
% 
Gr=zeros(2*ns,2*np);
G0=zeros(2*ns,1);
xs=[x1 x2];         % station x position
ys=[y1 y2];         % station y position
xp=[xp1 xp2];       % dyke parameter x position
yp=[yp1 yp2];       % dyke parameter y position

for i=1:2*ns          %loop on data
   for j=1:2*np       %loop on parameter
       
       %left dyke
       DZ=ys(i)-yp(j);
       DX=xs(i)-xp(j);
       C=DZ/(DX*DX+DZ*DZ);
       Gr(i,j)=alpha*C/2700;
       G0(i)=G0(i)-alpha*C;
   end
end

if (fig==1)
  figure(3);
  plot(xs,Gr*rhoD+G0,'.');grid
  title('gravity measurement (mugal) as function of distance');
  xlabel('meters')
  ylabel('mugal')
end

%=========================================================
%
% create a synthetic dataset
%
%=========================================================

% the right dyke is cold, the magma is rising on the left dyke
% In the left dyke, everything is cold from top down to 1000m
%
% cold dyke is 3000m/S and 2800kg/m3
% melt dyke is 1000m/s and 2000kg/m3

I=find(yp<1000 & xp<2500);
rhoD(I)=2000;
SpD(I)=1./1000;

if (fig==1)
  %update travel time figure with data
  figure(2)
  hold on;plot(y1,TtimeDyke*SpD+ttimeCone,'r.')
  %update gravity figure with data
  figure(3)
  hold on;plot(xs,Gr*rhoD+G0,'r.')
end
%=========================================================
%
% store all data: matrices and synthetic data
%
%=========================================================

% Create Data vector ordered as [ns traveltime; 2*ns gravity measurement]
% also substract constant values from vector to keep the system stricly
% linear
data=zeros(3*ns,1);
data(1:ns)=(TtimeDyke*SpD+ttimeCone) - ttimeCone;
data(ns+1:end)=(Gr*rhoD+G0) - G0;

modeltrue=zeros(4*np,1);
modeltrue(1:2*np)=SpD;
modeltrue(2*np+1:end)=rhoD;

% and save what we need for the inverse problem
if (octave==1)  
    save -binary inverseProblem.oct ns np data Gr TtimeDyke data ttimeCone G0 modeltrue y1 xs ys xp yp
else
    save -mat inverseProblem.mat ns np data Gr TtimeDyke data ttimeCone G0 modeltrue y1 xs ys xp yp
end


